<?php

namespace Drupal\migrate_file_field_bundle\Plugin\migrate\source;

use Drupal\Core\Database\Query\SelectInterface;
use Drupal\file\Plugin\migrate\source\d7\File;
use Drupal\migrate\Row;

/**
 * Get Drupal 7 files values by field and bundle.
 *
 * Available configuration keys:
 * - field: (required) The field machine name (string) to get values from.
 * - bundle: (required) The entity bundle (node type) the above field belongs to.
 *
 * Example:
 *
 * @code
 * source:
 *   plugin: file_field_bundle
 *   scheme: public
 *   constants:
 *   source_base_path: 'https://example.com/'
 *   field: field_my_file
 *   bundle: article
 * @endcode
 *
 * @MigrateSource(
 *   id = "file_field_bundle",
 *   source_module = "file"
 * )
 */
class FileFieldBundle extends File {

  /**
   * {@inheritdoc}
   */
  public function query(): SelectInterface {
    $query = parent::query();
    $extra_fields = $this->getExtraFields();

    // Filter by field machine name, if configured.
    $field = $this->configuration['field'] ?: NULL;
    // Filter by bundle (of the above field)
    $bundle = $this->configuration['bundle'] ?: NULL;

    if ($bundle !== NULL && $field !== NULL) {
      $table = "field_data_" . $field;
      $condition = 'fd.' . $field . '_fid = f.fid';

      $query->leftJoin($table, "fd", $condition);
      $query->condition('fd.bundle', $bundle);

      // Extra fields.
      foreach ($extra_fields as $ef) {
        $query->addField('fd', $ef);
      }
    }

    return $query;
  }

  /**
   * Extra fields coming from the Drupal field.
   *
   * @return string[]
   *   Array of fields
   */
  public function getExtraFields(): array {
    return [
      'entity_type',
      'bundle',
      'entity_id',
      'revision_id',
      'language',
      'delta',
      $this->configuration['field'] . '_display',
      $this->configuration['field'] . '_description',
    ];
  }

  /**
   * Extra fields but with descriptions.
   *
   * @return array
   *   Array of fields with descriptions
   */
  public function getExtraFieldsWithDescription(): array {
    $extra_fields = $this->getExtraFields();
    $extra_fields_with_description = [];

    foreach ($extra_fields as $extra_field) {
      $extra_fields_with_description[$extra_field] = $this->t("Extra field.");
    }

    return $extra_fields_with_description;
  }

  /**
   * {@inheritdoc}
   */
  public function fields(): array {
    $fields = parent::fields();
    return array_merge($fields, $this->getExtraFieldsWithDescription());
  }

  /**
   * {@inheritdoc}
   */
  public function getIds(): array {
    return [
      'fid' => [
        'type' => 'integer',
        'alias' => 'f',
      ],
      'entity_id' => [
        'type' => 'integer',
        'alias' => 'fd',
      ],
      'bundle' => [
        'type' => 'string',
        'alias' => 'fd',
      ],
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function prepareRow(Row $row): bool {
    $description_key = $this->configuration['field'] . '_description';
    $description = $row->getSourceProperty($description_key);
    if ($description) {
      $row->setSourceProperty('title', $description);
    }
    return parent::prepareRow($row);
  }

}
